#!/bin/bash

#
# WARNING: If you happen to edit this file then get the SHA256 using 'sha256sum setup_iptable_rules.sh' command
# and modify the isolateDeviceCommandHandler.cpp and UnioslateDeviceCommandHandler.cpp with the new SHA
#

MDE_CHAIN=mdechain
MDE_PACKET_STAMP="-m mark ! --mark 0x1/0x1"
MDE_NFQUEUE_BYPASS="NFQUEUE --queue-num 0 --queue-bypass"
MDE_TCP_PACKET_FILTER="-p tcp --tcp-flags FIN,SYN,RST,ACK,PSH SYN"
MDE_UDP_PACKET_FILTER="-p udp"
MDE_IPTABLES_SAVE_FILE="/opt/microsoft/mdatp/conf/mde_isolate_device_iptable_rules.txt"

if [ "$(id -u)" != "0" ] ; then
   logger -p daemon.err -t "setup_iptables_rules" "User has insufficient privilege"
   echo "setup_iptables_rules" "User has insufficient privilege"
   exit 4
fi

run_or_fail() {
   cmd="$@"
   status="0"
   out=$($cmd 2>&1) || status="$?"
   
   if [ "$status" != "0" ]; then
      cmd=$(echo $cmd | /bin/sed 's/[[:space:]]/ /g')
      logger -p daemon.err -t "setup_iptables_rules" "Failed to run: \"$cmd\", error code: $status, \"$out\""
      echo "setup_iptables_rules" "Failed to run: \"$cmd\", error code: $status, \"$out\""
      exit 4
   fi
}

run_or_warn() {
   cmd="$@"
   status="0"
   out=$($cmd 2>&1) || status="$?"
   
   if [ "$status" != "0" ]; then
      cmd=$(echo $cmd | /bin/sed 's/[[:space:]]/ /g')
      logger -p daemon.warn -t "cleanup_iptables_rules" "\"$cmd\" returns with code: $status, message: \"$out\""
      echo "cleanup_iptables_rules" "Failed to run: \"$cmd\", error code: $status, \"$out\""
      return 4
   fi
}

setup_iptable_rules() {
   echo "Setting up IPtables rules"
   if ! iptables -N ${MDE_CHAIN}; then
      echo "Iptable already setup"
      echo "IPtables rules setup completed"
      return 0
   fi
    
   ### Input chain rule ###
   # Intercept TCP inbound connection
   run_or_fail "iptables -I INPUT ${MDE_PACKET_STAMP} ${MDE_TCP_PACKET_FILTER} -j ${MDE_CHAIN}"
   #run_or_fail "iptables -I INPUT ${MDE_PACKET_STAMP} ${MDE_UDP_PACKET_FILTER} -j ${MDE_CHAIN}"

   # Allow inbound ssh
   run_or_fail "iptables -I INPUT -i eth0 -p tcp --dport 22 -m state --state NEW,ESTABLISHED -j ACCEPT"
   
   # Allow DNS packets
   run_or_fail "iptables -I INPUT --protocol udp --sport 53 -j ACCEPT"
   run_or_fail "iptables -I INPUT --protocol udp --dport 53 -j ACCEPT"

   ### Output chain rule ###
   # Intercept TCP outbound connection rules
   run_or_fail "iptables -I OUTPUT ${MDE_PACKET_STAMP} ${MDE_TCP_PACKET_FILTER} -j ${MDE_CHAIN}"
   #run_or_fail "iptables -I OUTPUT ${MDE_PACKET_STAMP} ${MDE_UDP_PACKET_FILTER} -j ${MDE_CHAIN}"

   # Allow inbound ssh connection traffic
   run_or_fail "iptables -I OUTPUT -o eth0 -p tcp --sport 22 -m state --state ESTABLISHED -j ACCEPT"

   # Allow packets owned by mdatp GID
   run_or_fail "iptables -I OUTPUT -m owner --gid-owner mdatp -j ACCEPT"

   # Allow DNS traffic
   run_or_fail "iptables -I OUTPUT --protocol udp --sport 53 -j ACCEPT"
   run_or_fail "iptables -I OUTPUT --protocol udp --dport 53 -j ACCEPT"

   # Reject all other traffic
   run_or_fail "iptables -A OUTPUT --protocol all -j REJECT --reject-with icmp-port-unreachable"

   run_or_fail "iptables -I ${MDE_CHAIN} -j ${MDE_NFQUEUE_BYPASS}"

   iptables -L
   echo "IPtables rules setup completed"
}

cleanup_iptable_rules() {
   echo "Flush IPtable rules"
   run_or_warn "iptables -D INPUT ${MDE_PACKET_STAMP} ${MDE_TCP_PACKET_FILTER} -j ${MDE_CHAIN}"
   #run_or_warn "iptables -D INPUT ${MDE_PACKET_STAMP} ${MDE_UDP_PACKET_FILTER} -j ${MDE_CHAIN}"
   run_or_warn "iptables -D INPUT -i eth0 -p tcp --dport 22 -m state --state NEW,ESTABLISHED -j ACCEPT"
   run_or_warn "iptables -D INPUT --protocol udp --sport 53 -j ACCEPT"
   run_or_warn "iptables -D INPUT --protocol udp --dport 53 -j ACCEPT"

   run_or_warn "iptables -D OUTPUT ${MDE_PACKET_STAMP} ${MDE_TCP_PACKET_FILTER} -j ${MDE_CHAIN}"
   #run_or_warn "iptables -D OUTPUT ${MDE_PACKET_STAMP} ${MDE_UDP_PACKET_FILTER} -j ${MDE_CHAIN}"
   run_or_warn "iptables -D OUTPUT --protocol all -j REJECT --reject-with icmp-port-unreachable"
   run_or_warn "iptables -D OUTPUT -o eth0 -p tcp --sport 22 -m state --state ESTABLISHED -j ACCEPT"
   run_or_warn "iptables -D OUTPUT -m owner --gid-owner mdatp -j ACCEPT"
   run_or_warn "iptables -D OUTPUT --protocol udp --sport 53 -j ACCEPT"
   run_or_warn "iptables -D OUTPUT --protocol udp --dport 53 -j ACCEPT"
   
   run_or_warn "iptables -D ${MDE_CHAIN} -j ${MDE_NFQUEUE_BYPASS}"
   run_or_warn "iptables -X ${MDE_CHAIN}"
   iptables -L
   echo "Cleaned up all IPtables rules"
}

if [ "$1" = "-i" ]; then
   setup_iptable_rules
elif [ "$1" = "-u" ]; then
   cleanup_iptable_rules
else
   echo "Parts of the command couldn't be recognized, including: '-flag'" > /dev/stderr
   exit 1
fi